// ApiView.cpp : implementation of the CApiBrowserView class
//

#include "stdafx.h"
#include "ApiBrow.h"

#include "ApiDoc.h"
#include "ApiView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CApiBrowserView

IMPLEMENT_DYNCREATE(CApiBrowserView, CListView)

BEGIN_MESSAGE_MAP(CApiBrowserView, CListView)
	//{{AFX_MSG_MAP(CApiBrowserView)
	ON_NOTIFY_REFLECT(LVN_GETDISPINFO, OnGetdispinfo)
	ON_NOTIFY_REFLECT(LVN_COLUMNCLICK, OnColumnClick)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CApiBrowserView construction/destruction

CApiBrowserView::CApiBrowserView()
{
	m_nLastItem = -1;
	m_pstrPool[0] =
	m_pstrPool[1] =
	m_pstrPool[2] = NULL;
	m_nNextFree = 0;
	m_nSortOrder = 0;
}

CApiBrowserView::~CApiBrowserView()
{
}

void CApiBrowserView::ResetContent()
{
	CListCtrl& theCtrl = GetListCtrl();
	theCtrl.DeleteAllItems();

	CHeaderCtrl* pHeader = (CHeaderCtrl*) theCtrl.GetDlgItem(0);
	ASSERT(pHeader != NULL);
	int nColCount = pHeader->GetItemCount();
	int nIndex;

	for (nIndex = nColCount-1; nIndex >= 0; nIndex--)
		theCtrl.DeleteColumn(nIndex);
}

BOOL CApiBrowserView::PreCreateWindow(CREATESTRUCT& cs)
{
	cs.style |= LVS_REPORT | WS_HSCROLL;
	cs.style &= ~(LVS_SORTASCENDING | LVS_SORTDESCENDING);
	return CListView::PreCreateWindow(cs);
}

int CALLBACK CompareAPIFunc(LPARAM lParam1, LPARAM lParam2, 
	LPARAM lParamSort)
{
	CApiBrowserView* pView = (CApiBrowserView*) lParamSort;
	ASSERT(pView != NULL);
	ASSERT_VALID(pView);

	CApiBrowserDoc* pDoc = pView->GetDocument();
	ASSERT(pDoc != NULL);
	ASSERT_VALID(pDoc);

	CString strItem1 = pDoc->m_Lines[lParam1];
	CString strItem2 = pDoc->m_Lines[lParam2];

	CString strField1 = pDoc->GetField(strItem1, pView->m_nSortOrder);
	CString strField2 = pDoc->GetField(strItem2, pView->m_nSortOrder);

	return strField1.Compare(strField2);
}

/////////////////////////////////////////////////////////////////////////////
// CApiBrowserView drawing

void CApiBrowserView::OnDraw(CDC* pDC)
{
	CApiBrowserDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
}

void CApiBrowserView::OnInitialUpdate()
{
	CWaitCursor wait;

	CListView::OnInitialUpdate();

	CListCtrl& theCtrl = GetListCtrl();
	CWindowDC dc(this);
	CFont* pOldFont;
	CFont* pTextFont = GetFont();
	CSize sHeader;
	int nCheckWidth = ::GetSystemMetrics(SM_CXMENUCHECK);

	CApiBrowserDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	int nColumns = pDoc->CountChars(pDoc->m_strColumns);
	int nIndex;
	CString strCurrent;
	for (nIndex = 0; nIndex < nColumns; nIndex++)
	{
		strCurrent = pDoc->GetField(pDoc->m_strColumns, nIndex);
		pOldFont = dc.SelectObject(pTextFont);
		sHeader = dc.GetTextExtent(strCurrent);
		dc.SelectObject(pOldFont);

		theCtrl.InsertColumn(nIndex, strCurrent, LVCFMT_LEFT, nCheckWidth + sHeader.cx);
	}

	int nCount = pDoc->m_Lines.GetSize();
	if (nCount == 0)
		return;

	CString strItem;

	LV_ITEM lvi;
	lvi.mask = LVIF_TEXT | LVIF_PARAM;
	lvi.iSubItem = 0;

	theCtrl.SetRedraw(FALSE);
	for (nIndex = 0; nIndex < nCount; nIndex++)
	{
		strCurrent = pDoc->m_Lines[nIndex];
		strItem = pDoc->GetField(strCurrent, 0);

		lvi.iItem = nIndex;
		lvi.lParam = nIndex;
		lvi.pszText = (LPTSTR) (LPCTSTR) strItem;

		theCtrl.InsertItem(&lvi);
	}


	CHeaderCtrl* pHeader = (CHeaderCtrl*) theCtrl.GetDlgItem(0);
	ASSERT(pHeader != NULL);
	int nColCount = pHeader->GetItemCount();

	TCHAR szBuffer[255];
	HD_ITEM hdi;
	hdi.mask = HDI_TEXT;
	hdi.pszText = szBuffer;
	hdi.cchTextMax = 255;

	for (nIndex = 0; nIndex < nColCount; nIndex++)
		pHeader->GetItem(nIndex, &hdi);

	theCtrl.SetRedraw(TRUE);
	theCtrl.InvalidateRect(NULL);

}

/////////////////////////////////////////////////////////////////////////////
// CApiBrowserView diagnostics

#ifdef _DEBUG
void CApiBrowserView::AssertValid() const
{
	CListView::AssertValid();
}

void CApiBrowserView::Dump(CDumpContext& dc) const
{
	CListView::Dump(dc);
}

CApiBrowserDoc* CApiBrowserView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CApiBrowserDoc)));
	return (CApiBrowserDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CApiBrowserView message handlers

void CApiBrowserView::OnGetdispinfo(NMHDR* pNMHDR, LRESULT* pResult) 
{
	LV_DISPINFO* pDispInfo = (LV_DISPINFO*)pNMHDR;

	if (pDispInfo->item.mask & LVIF_TEXT)
	{
		CApiBrowserDoc* pDoc = GetDocument();
		ASSERT_VALID(pDoc);

		if (m_nLastItem != pDispInfo->item.lParam)
		{
			m_nLastItem = pDispInfo->item.lParam;
			m_strItem = pDoc->m_Lines[m_nLastItem];
		}
		CString strField = pDoc->GetField(m_strItem, pDispInfo->item.iSubItem);

		LPTSTR pstrBuffer = AddPool(&strField);
		pDispInfo->item.pszText = pstrBuffer;
	}	
	*pResult = 0;
}

LPTSTR CApiBrowserView::AddPool(CString* pstr)
{
	LPTSTR pstrRetVal;
	int nOldest = m_nNextFree;
	if (nOldest == -1)
		nOldest = 2;

	m_strCPool[m_nNextFree] = *pstr;
	pstrRetVal = m_strCPool[m_nNextFree].LockBuffer();
	m_pstrPool[m_nNextFree++] = pstrRetVal;
	m_strCPool[nOldest].ReleaseBuffer();

	if (m_nNextFree == 3)
		m_nNextFree = 0;
	return pstrRetVal;
}

void CApiBrowserView::OnColumnClick(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_LISTVIEW* pNMListView = (NM_LISTVIEW*)pNMHDR;
	CListCtrl& theCtrl = GetListCtrl();
	
	m_nSortOrder = pNMListView->iSubItem;
	CWaitCursor waiter;
	theCtrl.SortItems(CompareAPIFunc, (LPARAM) this);
	*pResult = 0;
}
